# -*- coding: utf-8 -*-
from . import model
from .initializer import MetabaseInitializer
from sqlalchemy import create_engine
from sqlalchemy.exc import SQLAlchemyError

import argparse
import logging
import requests
import sys


log = logging.getLogger(__name__)


def get_metabase_args():
    parser = argparse.ArgumentParser(
        description=(
            "Initialize a metabase instance that has been freshly restored from a SQL "
            "dump by adapting settings to the given parameters."
        )
    )
    parser.add_argument(
        "--metabase-host",
        type=str,
        required=False,
        default="localhost",
        help=("Host that the metabase instance is running on"),
    )
    parser.add_argument(
        "--metabase-port",
        type=str,
        required=False,
        default=3000,
        help=("Port that the metabase instance is running on"),
    )
    parser.add_argument(
        "--metabase-user",
        type=str,
        required=True,
        help=("User name for connecting to the metabase instance"),
    )
    parser.add_argument(
        "--metabase-password",
        type=str,
        required=True,
        help=("Password for connecting to the metabase instance"),
    )
    parser.add_argument(
        "--database-name", type=str, help=("Name of the internal metabase database")
    )
    parser.add_argument(
        "--database-engine",
        type=str,
        required=False,
        default="postgres",
        help=("Database engine that the statistics database is running on"),
    )
    parser.add_argument(
        "--database-host",
        type=str,
        required=False,
        default="localhost",
        help=("Host that the postgresql server is running on"),
    )
    parser.add_argument(
        "--database-port",
        type=str,
        required=False,
        default=5432,
        help=("Port that the postgresql server is running on"),
    )
    parser.add_argument(
        "--database-user",
        type=str,
        required=True,
        help=("User name for connecting to the postgresql server"),
    )
    parser.add_argument(
        "--database-password",
        type=str,
        required=True,
        help=("Password for connecting to the postgresql server"),
    )
    parser.add_argument(
        "--database-name-statistics",
        type=str,
        help=("Name of the postgresql statistics database"),
    )
    parser.add_argument(
        "--database-pattern-statistics",
        type=str,
        default="statistics_{country}",
        help=(
            "Pattern for constructing the name of the postgresql statistics databases. "
            "{country} will be replaced by the two-letter country code. Default: "
            "statisticts_{country}"
        ),
    )
    parser.add_argument(
        "--statistics-user",
        type=str,
        action="append",
        nargs=4,
        metavar="USER_INFO",
        help=(
            "Email address, password, first and last name, separated by whitespace, "
            "for a non-superuser account to create for viewing the statistics"
        ),
    )
    parser.add_argument(
        "--global-statistics",
        action="store_true",
        help=("If passed, global-only dashboard cards will be added."),
    )
    parser.add_argument(
        "--countries",
        type=str,
        help=(
            "Comma separated list of country codes for which database, collection, "
            "dashboards and cards will be set up."
        ),
    )
    parser.add_argument(
        "--ldap-host",
        type=str,
        help=("LDAP host name or IP-address"),
    )
    parser.add_argument(
        "--ldap-port",
        type=str,
        help=("LDAP port"),
    )
    parser.add_argument(
        "--ldap-bind-dn",
        type=str,
        help=("LDAP bind DN"),
    )
    parser.add_argument(
        "--ldap-password",
        type=str,
        help=("LDAP password"),
    )
    parser.add_argument(
        "--ldap-user-base",
        type=str,
        help=("LDAP user base DN"),
    )
    parser.add_argument(
        "--ldap-user-filter",
        type=str,
        help=("LDAP user filter"),
    )
    parser.add_argument(
        "--ldap-attribute-firstname",
        type=str,
        default="givenName",
        help=("LDAP attribute to use as first name"),
    )
    return parser.parse_args()


def bootstrap_metabase_instance(args):
    api_url = "http://{args.metabase_host}:{args.metabase_port}".format(args=args)
    probe = requests.post(
        "{api_url}/api/session".format(api_url=api_url),
        json={
            "username": args.metabase_user,
            "password": args.metabase_password,
        },
    )
    if probe.ok:
        log.info("Metabase user already set up")
        return
    log.info("Setting up metabase user")
    result = requests.get(f"{api_url}/api/session/properties")
    token = result.json()["setup-token"]
    bootstrap = {
        "token": token,
        "prefs": {
            "site_name": "Syslab.com",
            "site_locale": "en",
            "allow_tracking": "false",
        },
        "database": None,
        "user": {
            "first_name": "Admin",
            "last_name": "Syslab",
            "email": args.metabase_user,
            "password": args.metabase_password,
            "site_name": "Syslab.com",
        },
    }
    result = requests.post(api_url + "/api/setup", json=bootstrap)
    if not result.ok:
        log.warn("Bootstrap returned %r: %r!", result.status_code, result.text)


def init_statistics_databases(args):
    names = []
    if args.global_statistics:
        names.append("global")
    if args.countries:
        names.extend([country.strip() for country in args.countries.split(",")])

    for country in names:
        log.info("Initializing %s database", country)
        database_name = args.database_pattern_statistics.format(country=country)
        # metabase expects "postgres", sqlalchemy "postgresql"
        database_engine = (
            "postgresql" if args.database_engine == "postgres" else args.database_engine
        )
        database_url = (
            f"{database_engine}://{args.database_user}:{args.database_password}@"
            f"{args.database_host}:{args.database_port}/{database_name}"
        )
        engine = create_engine(database_url)
        try:
            model.Base.metadata.create_all(bind=engine, checkfirst=True)
        except SQLAlchemyError as e:
            log.warning(f"Could not set up {database_name}: {e}")


def init_metabase_instance():
    logging.basicConfig(stream=sys.stderr, level=20)
    args = get_metabase_args()

    log.info("Initializing metabase instance")
    init_statistics_databases(args)
    bootstrap_metabase_instance(args)
    initializer = MetabaseInitializer(args)
    initializer()



from . import config
from .content import CardFactory
from .content import SectorCardFactory
from .metabase import OiraMetabase_API
from pkg_resources import resource_string
from time import sleep

import logging


log = logging.getLogger(__name__)


class MetabaseInitializer(object):
    _total_cols = 16

    def __init__(self, args):
        self.args = args
        self.engine = args.database_engine
        api_url = "http://{args.metabase_host}:{args.metabase_port}".format(args=args)
        self.mb = OiraMetabase_API(api_url, args.metabase_user, args.metabase_password)
        self._existing_items = None

    def __call__(self):
        self.mb.put("/api/setting/show-homepage-xrays", json={"value": False})
        self.mb.put("/api/setting/show-homepage-data", json={"value": False})
        for database in self.mb.get("/api/database").json()["data"]:
            if database["name"] == "Sample Dataset":
                self.mb.delete("/api/database/{}".format(database["id"]))

        self.set_up_start_here_dashboard()

        global_group_id = self.set_up_global_group()

        if self.args.global_statistics:
            global_database_id = self.set_up_database(engine=self.engine)
            global_collection_id = self.set_up_global_collection()
            self.set_up_account(
                database_id=global_database_id, collection_id=global_collection_id
            )
            self.set_up_assessment(
                database_id=global_database_id, collection_id=global_collection_id
            )
            self.set_up_tool(
                database_id=global_database_id, collection_id=global_collection_id
            )
            self.set_up_questionnaire(
                database_id=global_database_id, collection_id=global_collection_id
            )

        # `countries` has this format:
        # countries = {
        #     "de": {
        #         "group": 23,
        #         "database": 42,
        #         "collection": 69,
        #     },
        #     ...
        # }
        countries = {}
        if self.args.countries:
            countries = {
                country.strip(): {} for country in self.args.countries.split(",")
            }

            for country in countries:
                countries[country]["group"] = self.set_up_country_group(country)
                if not self.args.global_statistics:
                    countries[country]["database"] = self.set_up_database(
                        country=country, engine=self.engine
                    )
                    countries[country]["collection"] = self.set_up_country_collection(
                        country
                    )
                    self.set_up_account(
                        country=country,
                        database_id=countries[country]["database"],
                        collection_id=countries[country]["collection"],
                    )
                    self.set_up_assessment(
                        country=country,
                        database_id=countries[country]["database"],
                        collection_id=countries[country]["collection"],
                    )
                    self.set_up_tool(
                        country=country,
                        database_id=countries[country]["database"],
                        collection_id=countries[country]["collection"],
                    )
                    self.set_up_questionnaire(
                        country=country,
                        database_id=countries[country]["database"],
                        collection_id=countries[country]["collection"],
                    )
                    if country.upper() == "FR":
                        self.set_up_inrs(
                            country=country,
                            database_id=countries[country]["database"],
                            collection_id=countries[country]["collection"],
                        )

            if not self.args.global_statistics:
                self.set_up_country_permissions(countries, global_group_id)
            else:
                self.set_up_countries_overview(global_database_id, global_collection_id)

        # `sectors` has this format:
        # sectors = {
        #     "Cleaning": {
        #         "collection": 69,
        #         "cards": {
        #             "accumulated_assessments": {
        #                 "id": 42,
        #                 ...
        #             },
        #             ...
        #         },
        #     },
        #     ...
        # }
        sectors = {}
        if self.args.global_statistics:
            sectors = self.set_up_sectors(global_database_id, global_collection_id)
            self.set_up_global_permissions(
                global_database_id,
                countries,
                sectors,
                global_group_id,
                global_collection_id,
            )

        if self.args.ldap_host:
            self.set_up_ldap(countries, global_group_id)

        if self.args.statistics_user:
            users = self.mb.get("/api/user").json()["data"]
            user_emails = [user["email"] for user in users]
            for email, password, first_name, last_name in (
                self.args.statistics_user or []
            ):
                if email not in user_emails:
                    log.info("Creating user {}".format(email))
                    self.mb.post(
                        "/api/user",
                        json={
                            "first_name": first_name,
                            "last_name": last_name,
                            "email": email,
                            "password": password,
                            "group_ids": [1, 4],
                        },
                    )
                else:
                    log.info("Modifying user {}".format(email))
                    user_id = [user["id"] for user in users if user["email"] == email][
                        0
                    ]
                    self.mb.put(
                        "/api/user/{}".format(user_id),
                        json={
                            "first_name": first_name,
                            "last_name": last_name,
                            "email": email,
                            "password": password,
                            "group_ids": [1, 4],
                        },
                    )

        log.info("Done initializing metabase instance")

    def set_up_database(self, country=None, engine="postgres"):
        if country is None:
            db_name = "statistics_global"
        else:
            db_name = self.args.database_pattern_statistics.format(
                country=country.lower()
            )
        details = {
            "dbname": db_name,
        }
        if engine == "sqlite":
            details["db"] = f"/home/oira/statistics/var/{db_name}.sqlite"
        else:
            details.update(
                {
                    "host": self.args.database_host,
                    "port": self.args.database_port,
                    "user": self.args.database_user,
                    "password": self.args.database_password,
                }
            )
        db_data = {
            "name": db_name,
            "engine": engine,
            "details": details,
        }
        db_id = self.create("database", db_name, extra_data=db_data)

        self.mb.post("/api/database/{}/sync".format(db_id))
        while not [
            entry["msg"]
            for entry in self.mb.get("/api/util/logs").json()
            if "FINISHED: Sync {} Database {} '{}'".format(engine, db_id, db_name)
            in entry["msg"]
        ]:
            log.info("Waiting for database sync to finish...")
            sleep(1)

        db_info = self.mb.get(
            "/api/database/{}/metadata?include_hidden=true".format(db_id)
        ).json()
        for table_info in db_info["tables"]:
            self.mb.post(
                "/api/table/{}/rescan_values".format(table_info["id"]),
                json={},
            )
            self.mb.put(
                "/api/table/{}".format(table_info["id"]),
                json={"field_order": "database"},
            )
            for field_info in table_info["fields"]:
                if (
                    table_info["name"] in ["assessment", "company"]
                    and field_info["name"] == "id"
                    or (country is not None and field_info["name"] == "country")
                ):
                    self.mb.put(
                        "/api/field/{}".format(field_info["id"]),
                        json={"visibility_type": "sensitive"},
                    )

        return db_id

    def create(self, obj_type, obj_name, extra_data={}, reuse=True):
        if obj_type == "group":
            url = "/api/permissions/group"
        else:
            url = "/api/{}".format(obj_type)
        obj_data = {"name": obj_name}
        obj_data.update(extra_data)

        obj_exists = obj_name in self.existing_items[obj_type + "s"]
        if obj_exists:
            obj_id = self.existing_items[obj_type + "s"][obj_name]
            if reuse:
                log.info("Keeping existing {} '{}'".format(obj_type, obj_name))
                if extra_data:
                    obj_info = self.mb.put(
                        "{}/{}".format(url, obj_id),
                        json=obj_data,
                    ).json()
                else:
                    obj_info = self.mb.get("{}/{}".format(url, obj_id)).json()
            else:
                log.info("Deleting existing {} '{}'".format(obj_type, obj_name))
                self.mb.delete("{}/{}".format(url, obj_id))
        if not obj_exists or (obj_exists and not reuse):
            log.info("Adding {} '{}'".format(obj_type, obj_name))
            result = self.mb.post(
                url,
                json=obj_data,
            )
            obj_info = result.json()
            if not result.ok and "duplicate key" in obj_info.get("message", ""):
                # retry, this usually goes away by itself
                log.info('Retrying after "duplicate key" error')
                result = self.mb.post(
                    url,
                    json=obj_data,
                )
                obj_info = result.json()
        obj_id = obj_info["id"]
        return obj_id

    def set_up_start_here_dashboard(self):
        intro_text = resource_string(__package__, "resources/intro_text.md").decode(
            "utf-8"
        )
        dashboard_data = {
            "description": "Introduction to the statistics",
            "collection_position": 1,
            "collection_id": None,
        }
        dashboard_id = self.create(
            "dashboard", "-> Start here", extra_data=dashboard_data
        )

        existing_cards = self.mb.get("/api/dashboard/{}".format(dashboard_id)).json()[
            "ordered_cards"
        ]
        if len(existing_cards) > 0:
            card_id = existing_cards[0]["id"]
        else:
            new_card = self.mb.post(
                "/api/dashboard/{}/cards".format(dashboard_id),
                json={"cardId": None},
            ).json()
            card_id = new_card["id"]

        intro_card = {
            "id": card_id,
            "card_id": None,
            "parameter_mappings": [],
            "series": [],
            "visualization_settings": {
                "virtual_card": {
                    "archived": False,
                    "dataset_query": {},
                    "name": None,
                    "display": "text",
                    "visualization_settings": {},
                },
                "text": intro_text,
            },
            "dashboard_id": dashboard_id,
            "size_x": 8,
            "size_y": 9,
            "col": 0,
            "row": 0,
        }
        self.mb.put(
            "/api/dashboard/{}/cards".format(dashboard_id),
            json={"cards": [intro_card]},
        )

    def set_up_global_group(self):
        return self.create("group", "global")

    def set_up_country_group(self, country):
        return self.create("group", country.upper())

    def set_up_global_collection(self):
        return self.create("collection", "-Global-", extra_data={"color": "#0000FF"})

    def set_up_country_collection(self, country):
        return self.create(
            "collection", country.upper(), extra_data={"color": "#00FF00"}
        )

    def set_up_global_permissions(
        self,
        global_database_id,
        countries,
        sectors,
        global_group_id,
        global_collection_id,
    ):
        log.info("Setting up global permissions")
        all_users_id = str(self.existing_items["groups"]["All Users"])

        # Database permissions
        permissions = self.mb.get("/api/permissions/graph").json()
        permissions["groups"].update(
            dict(
                {
                    str(all_users_id): {
                        str(global_database_id): {"data": {"schemas": "none"}},
                    },
                    str(global_group_id): {
                        str(global_database_id): {"data": {"schemas": "all"}},
                    },
                },
                **{
                    str(country_info["group"]): {
                        str(global_database_id): {"data": {"schemas": "all"}},
                    }
                    for country_info in countries.values()
                },
            )
        )

        self.mb.put("/api/permissions/graph", json=permissions)

        # Collection permissions
        collection_permissions = self.mb.get("/api/collection/graph").json()
        collection_permissions["groups"].update(
            dict(
                {
                    str(all_users_id): dict(
                        {
                            str(global_collection_id): "none",
                        },
                        **{
                            str(sector["collection"]): "none"
                            for sector in sectors.values()
                        },
                    ),
                    str(global_group_id): dict(
                        {
                            str(global_collection_id): "read",
                        },
                        **{
                            str(sector["collection"]): "read"
                            for sector in sectors.values()
                        },
                    ),
                },
                **{
                    str(country_info["group"]): dict(
                        {
                            str(global_collection_id): "read",
                        },
                        **(
                            {
                                str(sector["collection"]): "read"
                                for sector in sectors.values()
                            }
                            if country_id == "eu"
                            else {}
                        ),
                    )
                    for country_id, country_info in countries.items()
                },
            )
        )

        self.mb.put("/api/collection/graph", json=collection_permissions)

    def set_up_country_permissions(self, countries, global_group_id):
        log.info("Setting up country permissions")
        all_users_id = str(self.existing_items["groups"]["All Users"])

        # Database permissions
        permissions = self.mb.get("/api/permissions/graph").json()
        permissions["groups"].update(
            dict(
                {
                    str(all_users_id): {
                        str(country_info["database"]): {"data": {"schemas": "none"}}
                        for country_info in countries.values()
                    },
                    str(global_group_id): {
                        str(country_info["database"]): {"data": {"schemas": "all"}}
                        for country_info in countries.values()
                    },
                },
                **{
                    str(country_info["group"]): dict(
                        {
                            str(country_info["database"]): {"data": {"schemas": "all"}},
                        },
                        **{
                            str(country_other["database"]): {
                                "data": {"schemas": "none"}
                            }
                            for country_other in countries.values()
                            if country_info["group"] != country_other["group"]
                        },
                    )
                    for country_info in countries.values()
                },
            )
        )
        self.mb.put("/api/permissions/graph", json=permissions)

        # Collection permissions
        collection_permissions = self.mb.get("/api/collection/graph").json()
        collection_permissions["groups"].update(
            dict(
                {
                    str(all_users_id): {
                        str(country_info["collection"]): "none"
                        for country_info in countries.values()
                    },
                    str(global_group_id): {
                        str(country_info["collection"]): "read"
                        for country_info in countries.values()
                    },
                },
                **{
                    str(country_info["group"]): dict(
                        {
                            str(country_info["collection"]): "read",
                        },
                        **{
                            str(country_other["collection"]): "none"
                            for country_other in countries.values()
                            if country_info["group"] != country_other["group"]
                        },
                    )
                    for country_info in countries.values()
                },
            )
        )

        self.mb.put("/api/collection/graph", json=collection_permissions)

    def set_up_dashboard(
        self,
        dashboard_name=None,
        description=None,
        cards=[],
        country=None,
        database_id=None,
        collection_id=None,
        collection_position=None,
    ):
        if country is not None:
            dashboard_name = "{} ({})".format(dashboard_name, country.upper())

        dashboard_data = {
            "collection_id": collection_id,
            "collection_position": collection_position or 1,
        }
        dashboard_id = self.create(
            "dashboard", dashboard_name, extra_data=dashboard_data
        )

        log.info("Adding {} cards".format(dashboard_name))

        col = 0
        row = 0
        row_height = 4
        ordered_cards = self.mb.get("/api/dashboard/{}".format(dashboard_id)).json()[
            "ordered_cards"
        ]
        cards_is = {card["card_id"]: card["id"] for card in ordered_cards}
        cards_add = []
        cards_update = []
        cards_should = []
        # TODO: delete description if it is None but a card exists
        if description is not None:
            description_card = next(
                (card for card in ordered_cards if card["card_id"] is None), None
            )
            if not description_card:
                description_card = self.mb.post(
                    "/api/dashboard/{}/cards".format(dashboard_id),
                    json={"cardId": None},
                ).json()
            dashcard = {
                "id": description_card["id"],
                "card_id": None,
                "parameter_mappings": [],
                "series": [],
                "visualization_settings": {
                    "virtual_card": {
                        "archived": False,
                        "dataset_query": {},
                        "name": None,
                        "display": "text",
                        "visualization_settings": {},
                    },
                    "text": description,
                },
                "dashboard_id": dashboard_id,
                "size_x": 4,
                "size_y": 4,
                "col": col,
                "row": row,
            }
            col += 4
            row += 4
            cards_should.append(dashcard["card_id"])
            cards_update.append(dashcard)

        for card in cards:
            if "id" in card:
                card_id = card["id"]
            else:
                card_id = self.create("card", card["name"], extra_data=card)
            width = min(card.get("width", 4), self._total_cols)
            height = card.get("height", 4)
            if width + col > self._total_cols:
                col = 0
                row += row_height
                row_height = height
            else:
                row_height = max(height, row_height)
            dashcard = {
                "col": col,
                "row": row,
                "size_x": width,
                "size_y": height,
            }
            cards_should.append(card_id)
            if card_id not in cards_is:
                dashcard["cardId"] = card_id
                cards_add.append(dashcard)
            else:
                dashcard["card_id"] = card_id
                dashcard["id"] = cards_is[card_id]
                cards_update.append(dashcard)

            col += width

        cards_delete = [
            dashcard_id
            for card_id, dashcard_id in cards_is.items()
            if card_id not in cards_should
        ]
        for dashcard_id in cards_delete:
            self.mb.delete(
                "/api/dashboard/{}/cards?dashcardId={}".format(
                    dashboard_id, dashcard_id
                )
            )
        for dashcard in cards_add:
            self.mb.post(
                "/api/dashboard/{}/cards".format(dashboard_id),
                json=dashcard,
            )
        if cards_update:
            self.mb.put(
                "/api/dashboard/{}/cards".format(dashboard_id),
                json={"cards": cards_update},
            )
        return dashboard_id

    def set_up_account(self, country=None, database_id=34, collection_id=4):
        card_factory = CardFactory(self.mb, database_id, collection_id, country=country)
        cards = [
            card_factory.accumulated_users_per_type,
            card_factory.new_users_per_month,
            card_factory.user_conversions_per_month,
            card_factory.accumulated_registered_users_per_type,
            card_factory.accumulated_registered_users_over_time,
            card_factory.newsletter_subscriptions,
        ]
        self.set_up_dashboard(
            dashboard_name="Users Dashboard",
            description=(
                "## About this dashboard\n\n"
                "Shows all data in relation to users (registered users and guest users)"
                "\n\n"
                "Information on guest users before January 2020 is not available."
            ),
            cards=cards,
            country=country,
            database_id=database_id,
            collection_id=collection_id,
            collection_position=1,
        )

    def set_up_assessment(self, country=None, database_id=34, collection_id=3):
        assessments_card_factory = CardFactory(
            self.mb, database_id, collection_id, country=country
        )
        cards = [
            assessments_card_factory.accumulated_assessments,
            assessments_card_factory.new_assessments_per_month,
            assessments_card_factory.completion_of_assessments,
            assessments_card_factory.accumulated_assessments_over_time,
        ]
        if country is not None:
            cards.extend(
                [
                    assessments_card_factory.tools_by_assessment_completion,
                ]
            )
        description = (
            "## About this dashboard\n\nThis dashboard shows data in relation to the "
            "assessments done with the tools in {}, such as new "
            "assessments per month, accumulated assessments, qualitative indicators "
            "(how many of the questions in a tool have been answered) as well as the "
            "number of assessments by tool. Remember to use the filter options (upper "
            "right hand corner) if you want to have more detailed information about a "
            "tool (e.g. for Accumulated Assessments Over Time you can filter by "
            "tool/sector and by completion percentage).\n\n"
            "In order to see the tools that are accumulated under “others” in the "
            "cards that include tools information, you need to click on the table icon "
            "at the bottom centre of the screen.\n\n"
            "![Switch between table and chart]"
            "(/statistics/images/switch_table_graph.png)".format(
                "the current country" if country is not None else "OiRA"
            )
        )
        self.set_up_dashboard(
            dashboard_name="Assessments Dashboard",
            description=description,
            cards=cards,
            country=country,
            database_id=database_id,
            collection_id=collection_id,
            collection_position=2,
        )

    def set_up_tool(self, country=None, database_id=34, collection_id=None):
        tools_card_factory = CardFactory(
            self.mb, database_id, collection_id, country=country
        )
        cards = [
            tools_card_factory.top_tools_by_number_of_users,
            tools_card_factory.number_of_users_and_assessments_by_tool,
        ]
        description = (
            "## About this dashboard\n\n"
            "Compares tools in {} to each other using the metrics “started "
            "assessments” and “number of users who have used the tool”. "
            "In order to see the tools that are accumulated under “others” you can "
            "click on the table icon at the bottom centre of the screen.\n\n"
            "![Switch between table and chart]"
            "(/statistics/images/switch_table_graph.png)\n\n"
            "The *number of users by yearly average* is used as a rough measure of how "
            "popular a tool stays over time. Instead of taking the total number of "
            "users, the time a tool has been available is taken into account. "
            "This way a tool that was once popular but has fallen into disuse moves "
            "down in the ranking as time passes. Please note that this can mean that a "
            "tool with more total users can end up lower in the ranking than one with "
            "fewer total users. The reason is that the number of years "
            "is rounded up to the next whole number. E.g. a tool with 12 total "
            "users that has been online for just over a year will be treated as having "
            "run for two years and thus have 6 users per year, while a tool with 10 "
            "total users that has been online for just under a year will have 10 users "
            "per year.".format("the current country" if country is not None else "OiRA")
        )
        self.set_up_dashboard(
            dashboard_name="Tools Dashboard",
            description=description,
            cards=cards,
            country=country,
            database_id=database_id,
            collection_id=collection_id,
            collection_position=3,
        )

    def set_up_questionnaire(self, country=None, database_id=34, collection_id=None):
        card_factory = CardFactory(self.mb, database_id, collection_id, country=country)
        cards = [
            card_factory.number_of_survey_responses,
            card_factory.employees,
            card_factory.conductor,
            card_factory.referer if country != "fr" else card_factory.referer_france,
            card_factory.workers_participated,
            card_factory.needs_met,
            card_factory.recommend_tool,
        ]
        description = (
            "## About this dashboard\n\n"
            "Gives information about the answers to the voluntary questionnaire in the "
            "OiRA tool. This includes e.g. number of employees in the company, needs "
            "were met by the tool, etc. From April 2021 on this data can be filtered "
            "for certain time periods. Before April 2021 this information is not "
            "available.\n\n"
            "By using the “Tool Path” filter you can also filter the information from "
            "the questionnaire for different tools."
        )
        self.set_up_dashboard(
            dashboard_name="Questionnaire Dashboard",
            description=description,
            cards=cards,
            country=country,
            database_id=database_id,
            collection_id=collection_id,
            collection_position=4,
        )

    def set_up_sectors(self, global_database_id, global_collection_id):
        sectors = {}
        for sector_name in config.sectors:
            log.info("Adding sector {}".format(sector_name))
            sectors[sector_name] = {}
            collection_id = self.create(
                "collection",
                "Sector: {}".format(sector_name),
                extra_data={
                    "color": "#509EE3",
                },
            )
            sectors[sector_name]["collection"] = collection_id
            card_factory = SectorCardFactory(
                sector_name, self.mb, global_database_id, collection_id
            )
            cards = {
                "accumulated_assessments": card_factory.accumulated_assessments,
                "new_assessments_per_month": card_factory.new_assessments_per_month,
                "completion_of_assessments": card_factory.completion_of_assessments,
                "top_tools_by_number_of_assessments": card_factory.top_tools_by_number_of_assessments,
                "accumulated_assessments_over_time": card_factory.accumulated_assessments_over_time,
                "accumulated_number_of_users": card_factory.accumulated_number_of_users,
            }
            for card_token, card in cards.items():
                cards[card_token]["id"] = self.create(
                    "card", card["name"], extra_data=card
                )

            sectors[sector_name]["cards"] = cards
            sector_dashboard_id = self.set_up_dashboard(
                dashboard_name="Assessments ({})".format(sector_name),
                cards=list(cards.values())[:-2],
                database_id=global_database_id,
                collection_id=collection_id,
                collection_position=1,
            )
            self.mb.post(
                "/api/dashboard/{}/cards".format(sector_dashboard_id),
                json={
                    "cardId": cards["accumulated_assessments_over_time"]["id"],
                    "col": 4,
                    "row": 4,
                    "size_x": 8,
                    "size_y": 4,
                    "series": [
                        {
                            "id": cards["accumulated_number_of_users"]["id"],
                            "display": "line",
                        },
                    ],
                    "visualization_settings": {
                        "graph.show_trendline": False,
                        "graph.y_axis.title_text": "Number of Assessments/Users",
                        "graph.show_values": True,
                        "graph.x_axis.title_text": "Date",
                        "card.title": "Accumulated Assessments And Users ({})".format(
                            sector_name
                        ),
                        "series_settings": {
                            "count": {
                                "display": "line",
                                "title": "Accumulated Assessments",
                            },
                            "Accumulated Users Over Time ({})".format(sector_name): {
                                "title": "Accumulated Users"
                            },
                        },
                        "graph.label_value_frequency": "fit",
                        "graph.metrics": ["count"],
                        "graph.y_axis.auto_range": True,
                        "graph.y_axis.auto_split": False,
                        "graph.dimensions": ["start_date"],
                        "stackable.stack_type": None,
                    },
                },
            )

        overview_dashboard_id = self.set_up_dashboard(
            dashboard_name="Sectors Overview Dashboard",
            database_id=global_database_id,
            collection_id=global_collection_id,
            collection_position=5,
        )
        overview_cards = [
            {
                "token": "accumulated_assessments_over_time",
                "base_title": "Accumulated Assessments Over Time",
                "graph.dimensions": ["start_date"],
            },
            {
                "token": "completion_of_assessments",
                "base_title": "Completion of Assessments",
                "graph.dimensions": ["completion", "count"],
            },
        ]
        for idx, card_info in enumerate(overview_cards):
            combined_card = None
            for sector_name, sector_info in sectors.items():
                next_card = sector_info["cards"][card_info["token"]]
                if combined_card is None:
                    combined_card = {
                        "cardId": next_card["id"],
                        "col": 0,
                        "row": idx * 8,
                        "size_x": self._total_cols,
                        "size_y": 8,
                        "series": [],
                        "visualization_settings": {
                            "graph.dimensions": card_info["graph.dimensions"],
                            "graph.metrics": ["count"],
                            "series_settings": {
                                "count": {
                                    "color": "#A989C5",
                                    "display": "line",
                                    "title": sector_name,
                                },
                            },
                            "card.title": "{} Per Sector".format(
                                card_info["base_title"]
                            ),
                            "graph.show_trendline": False,
                        },
                    }
                else:
                    combined_card["series"].append(
                        {
                            "id": next_card["id"],
                        }
                    )
                    combined_card["visualization_settings"]["series_settings"][
                        "{} ({})".format(card_info["base_title"], sector_name)
                    ] = {
                        "title": sector_name,
                    }
            self.mb.post(
                "/api/dashboard/{}/cards".format(overview_dashboard_id),
                json=combined_card,
            )
        return sectors

    def set_up_inrs(self, country=None, database_id=34, collection_id=3):
        card_factory = CardFactory(self.mb, database_id, collection_id, country=country)
        cards = {
            token: getattr(card_factory, token)
            for token in [
                "oiras_poissonerie_sessions_cumulees",
                "oiras_boulangerie_sessions_cumulees",
                "oiras_boucherie_charcuterie_sessions_cumulees",
                "oiras_commerce_alimentaire_de_proximite_sessions_cumulees",
            ]
        }
        for card_token, card in cards.items():
            cards[card_token]["id"] = self.create("card", card["name"], extra_data=card)

        dashboard_id = self.set_up_dashboard(
            dashboard_name="INRS",
            cards=list(cards.values()),
            country=country,
            database_id=database_id,
            collection_id=collection_id,
            collection_position=5,
        )
        combined_card = None
        for idx, card in enumerate(cards.values()):
            if combined_card is None:
                combined_card = {
                    "cardId": card["id"],
                    "col": 0,
                    "row": 4,
                    "size_x": self._total_cols,
                    "size_y": 8,
                    "series": [],
                    "visualization_settings": {
                        "graph.dimensions": ["start_date"],
                        "graph.metrics": ["count"],
                        "series_settings": {
                            "count": {
                                "color": "#A989C5",
                                "display": "line",
                                "title": card["name"],
                            },
                        },
                        "graph.show_trendline": False,
                    },
                }
            else:
                combined_card["series"].append(
                    {
                        "id": card["id"],
                    }
                )
        self.mb.post(
            "/api/dashboard/{}/cards".format(dashboard_id),
            json=combined_card,
        )

    def set_up_countries_overview(self, global_database_id, global_collection_id):
        overview_dashboard_countryid = self.set_up_dashboard(
            dashboard_name="Countries Overview Dashboard",
            database_id=global_database_id,
            collection_id=global_collection_id,
            collection_position=6,
        )

        card_factory = CardFactory(self.mb, global_database_id, global_collection_id)
        card = card_factory.top_assessments_by_country
        card_id = self.create("card", card["name"], extra_data=card)
        self.mb.post(
            "/api/dashboard/{}/cards".format(overview_dashboard_countryid),
            json={
                "cardId": card_id,
                "col": 0,
                "row": 0,
                "size_x": 18,
                "size_y": 4,
            },
        )
        card = card_factory.accumulated_assessments_per_country
        card_id = self.create("card", card["name"], extra_data=card)
        self.mb.post(
            "/api/dashboard/{}/cards".format(overview_dashboard_countryid),
            json={
                "cardId": card_id,
                "col": 0,
                "row": 4,
                "size_x": 18,
                "size_y": 4,
            },
        )

        card = card_factory.registered_users_per_country
        card_id = self.create("card", card["name"], extra_data=card)
        self.mb.post(
            "/api/dashboard/{}/cards".format(overview_dashboard_countryid),
            json={
                "cardId": card_id,
                "col": 0,
                "row": 8,
                "size_x": 18,
                "size_y": 4,
            },
        )

    def set_up_ldap(self, countries, global_group_id):
        log.info("Setting up LDAP")
        group_mappings = {
            "cn={},ou=Countries,ou=OiRA_CMS,ou=Sites,dc=osha,dc=europa,dc=eu".format(
                country
            ): [info["group"]]
            for country, info in countries.items()
        }
        group_mappings["cn=ADMIN,ou=OiRA_CMS,ou=Sites,dc=osha,dc=europa,dc=eu"] = [
            global_group_id
        ]
        self.mb.put(
            "/api/ldap/settings",
            json={
                "ldap-enabled": True,
                "ldap-host": self.args.ldap_host,
                "ldap-port": self.args.ldap_port or "389",
                "ldap-bind-dn": self.args.ldap_bind_dn,
                "ldap-password": self.args.ldap_password,
                "ldap-user-base": self.args.ldap_user_base,
                "ldap-user-filter": self.args.ldap_user_filter,
                "ldap-attribute-firstname": self.args.ldap_attribute_firstname,
                "ldap-group-sync": True,
                "ldap-group-base": ("ou=OiRA_CMS,ou=Sites,dc=osha,dc=europa,dc=eu"),
            },
        )
        self.mb.put("/api/setting/ldap-group-mappings", json={"value": group_mappings})

    @property
    def existing_items(self):
        if not self._existing_items:
            self._existing_items = {}
            self._existing_items["groups"] = {
                group["name"]: group["id"]
                for group in self.mb.get("/api/permissions/group").json()
            }
            self._existing_items["databases"] = {
                db["name"]: db["id"]
                for db in self.mb.get("/api/database").json()["data"]
            }
            self._existing_items["collections"] = {
                collection["name"]: collection["id"]
                for collection in self.mb.get("/api/collection").json()
            }
            self._existing_items["dashboards"] = {
                dashboard["name"]: dashboard["id"]
                for dashboard in self.mb.get("/api/dashboard").json()
            }
            self._existing_items["cards"] = {
                card["name"]: card["id"] for card in self.mb.get("/api/card").json()
            }

        return self._existing_items



from . import config

import logging


log = logging.getLogger(__name__)


class CardFactory(object):
    extra_filter = None
    _raw_cards = {}

    def __init__(self, mb, database_id, collection_id, country=None):
        self.database_id = database_id
        self.collection_id = collection_id
        self.country = country
        database = mb.get(
            "/api/database/{}?include=tables.fields".format(database_id)
        ).json()
        self.tables = {}
        for table in database["tables"]:
            self.tables[table["name"]] = {
                "id": table["id"],
                "fields": {field["name"]: field["id"] for field in table["fields"]},
            }

    def __getattr__(self, name):
        if name not in self._raw_cards:
            return
        card = self.get_base_properties()
        card_data = self._raw_cards[name]
        card.update(card_data)
        card["name"] = self.transform_name(card["name"])
        if self.extra_filter:
            query_type = card["query_type"]
            if query_type == "query":
                if "source-query" in card["dataset_query"][query_type]:
                    orig_query = card["dataset_query"][query_type]["source-query"]
                else:
                    orig_query = card["dataset_query"][query_type]
                if "filter" in orig_query:
                    orig_filter = orig_query["filter"]
                    new_filter = [
                        "and",
                        orig_filter,
                        self.extra_filter[query_type],
                    ]
                else:
                    new_filter = self.extra_filter[query_type]
                orig_query["filter"] = new_filter

            elif query_type == "native":
                query = card["dataset_query"][query_type]["query"]
                if self.extra_filter[query_type] not in query:
                    log.warning(
                        "Filter not found in query: {}".format(
                            self.extra_filter[query_type]
                        )
                    )
            else:
                log.warning("Unknown query type {}".format(query_type))
        return card

    def get_base_properties(self):
        base = {
            "collection_id": self.collection_id,
            "database_id": self.database_id,
        }
        return base

    def transform_name(self, name):
        if self.country:
            return "{} ({})".format(name, self.country.upper())
        else:
            return name

    @property
    def _raw_cards(self):
        return {
            "accumulated_users_per_type": {
                "name": "Accumulated Users per Type",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["account"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ]
                        ],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Account Type",
                        "name": "account_type",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.show_legend": True,
                    "pie.show_legend_perecent": True,
                    "pie.colors": {
                        "converted": "#98D9D9",
                        "full": "#7172AD",
                        "guest": "#F9D45C",
                    },
                },
            },
            "accumulated_registered_users_per_type": {
                "name": "Accumulated Registered Users per Type",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["account"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ]
                        ],
                        "filter": [
                            "!=",
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ],
                            "guest",
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Account Type",
                        "name": "account_type",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.show_legend": True,
                    "pie.show_legend_perecent": True,
                    "pie.colors": {
                        "converted": "#98D9D9",
                        "full": "#7172AD",
                        "guest": "#F9D45C",
                    },
                },
            },
            "new_users_per_month": {
                "name": "New Users per Month",
                "display": "bar",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["account"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "datetime-field",
                                [
                                    "field-id",
                                    self.tables["account"]["fields"]["creation_date"],
                                ],
                                "month",
                            ],
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ],
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Creation Date",
                        "name": "creation_date",
                        "unit": "month",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/Text",
                        "display_name": "Account Type",
                        "name": "account_type",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.show_goal": False,
                    "graph.show_trendline": True,
                    "graph.y_axis.title_text": "Number of New Users",
                    "graph.show_values": True,
                    "stackable.stack_display": "bar",
                    "graph.x_axis.title_text": "Creation Date",
                    "graph.y_axis.auto_split": False,
                    "graph.metrics": ["count"],
                    "graph.label_value_formatting": "auto",
                    "series_settings": {"guest": {"color": "#F9D45C"}},
                    "graph.dimensions": ["creation_date", "account_type"],
                    "stackable.stack_type": None,
                },
            },
            "user_conversions_per_month": {
                "name": "User Conversions per Month",
                "display": "line",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["account"]["id"],
                        "filter": [
                            "=",
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ],
                            "converted",
                        ],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "datetime-field",
                                [
                                    "field-id",
                                    self.tables["account"]["fields"]["creation_date"],
                                ],
                                "month",
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Creation Date",
                        "name": "creation_date",
                        "unit": "month",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.x_axis.title_text": "Date",
                    "graph.dimensions": ["creation_date"],
                    "graph.metrics": ["count"],
                    "graph.show_values": True,
                    "series_settings": {
                        "count": {
                            "title": "Number of User Accounts Converted",
                            "color": "#98D9D9",
                        }
                    },
                },
            },
            "accumulated_registered_users_over_time": {
                "name": "Accumulated Registered Users Over Time",
                "display": "line",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["account"]["id"],
                        "filter": [
                            "=",
                            [
                                "field-id",
                                self.tables["account"]["fields"]["account_type"],
                            ],
                            "full",
                            "converted",
                        ],
                        "aggregation": [["cum-count"]],
                        "breakout": [
                            [
                                "datetime-field",
                                [
                                    "field-id",
                                    self.tables["account"]["fields"]["creation_date"],
                                ],
                                "month",
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Creation Date",
                        "name": "creation_date",
                        "unit": "month",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.dimensions": ["creation_date"],
                    "graph.metrics": ["count"],
                    "series_settings": {"count": {"color": "#A989C5"}},
                },
            },
            "newsletter_subscriptions": {
                "name": "Newsletter Subscriptions",
                "display": "table",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {"source-table": self.tables["newsletter"]["id"]},
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "description": None,
                        "semantic_type": "type/PK",
                        "coercion_strategy": None,
                        "name": "zodb_path",
                        "settings": None,
                        "field_ref": ["field", 120, None],
                        "effective_type": "type/Text",
                        "id": 120,
                        "visibility_type": "normal",
                        "display_name": "Path",
                        "fingerprint": None,
                        "base_type": "type/Text",
                    },
                    {
                        "description": None,
                        "semantic_type": "type/Quantity",
                        "coercion_strategy": None,
                        "name": "count",
                        "settings": None,
                        "field_ref": ["field", 121, None],
                        "effective_type": "type/Integer",
                        "id": 121,
                        "visibility_type": "normal",
                        "display_name": "Count",
                        "base_type": "type/Integer",
                    },
                ],
                "visualization_settings": {
                    "table.pivot_column": "count",
                    "table.cell_column": "zodb_path",
                    "column_settings": {
                        '["ref",["field",120,null]]': {"column_title": "Path"}
                    },
                },
                "height": 8,
                "width": 6,
            },
            "top_tools_by_number_of_users": {
                "name": "Top Tools by Number of Users / By yearly average",
                "display": "row",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["tool"]["id"],
                        "expressions": {
                            "Users per Year": [
                                "/",
                                [
                                    "field-id",
                                    self.tables["tool"]["fields"]["num_users"],
                                ],
                                [
                                    "+",
                                    [
                                        "field-id",
                                        self.tables["tool"]["fields"]["years_online"],
                                    ],
                                    1,
                                ],
                            ]
                        },
                        "order-by": [["desc", ["expression", "Users per Year"]]],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Tool Path",
                        "name": "Tool_path",
                        "special_type": "type/PK",
                    },
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Published Date",
                        "name": "published_date",
                        "unit": "default",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/Integer",
                        "display_name": "Years Online",
                        "name": "years_online",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/Integer",
                        "display_name": "Num Users",
                        "name": "num_users",
                        "special_type": "type/Quantity",
                    },
                    {
                        "base_type": "type/Float",
                        "display_name": "Users per Year",
                        "name": "Users per Year",
                        "special_type": None,
                    },
                ],
                "visualization_settings": {
                    "graph.dimensions": ["tool_path"],
                    "graph.metrics": ["Users per Year"],
                },
                "height": 8,
                "width": 6,
            },
            "number_of_users_and_assessments_by_tool": {
                "name": "Number Of Users And Assessments By Tool / Since publication",
                "display": "table",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["tool"]["id"],
                        "expressions": {
                            "Users per Year": [
                                "/",
                                [
                                    "field-id",
                                    self.tables["tool"]["fields"]["num_users"],
                                ],
                                [
                                    "+",
                                    [
                                        "field-id",
                                        self.tables["tool"]["fields"]["years_online"],
                                    ],
                                    1,
                                ],
                            ]
                        },
                        "fields": [
                            ["field-id", self.tables["tool"]["fields"]["tool_path"]],
                            ["field-id", self.tables["tool"]["fields"]["num_users"]],
                            [
                                "field-id",
                                self.tables["tool"]["fields"]["num_assessments"],
                            ],
                        ],
                        "order-by": [
                            [
                                "desc",
                                [
                                    "field-id",
                                    self.tables["tool"]["fields"]["num_assessments"],
                                ],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Tool Path",
                        "name": "tool_path",
                        "special_type": "type/PK",
                    },
                    {
                        "base_type": "type/Integer",
                        "display_name": "Num Users",
                        "name": "num_users",
                        "special_type": "type/Quantity",
                    },
                    {
                        "base_type": "type/Integer",
                        "display_name": "Num Assessments",
                        "name": "num_assessments",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "table.pivot": False,
                    "table.pivot_column": "num_users",
                    "table.cell_column": "num_assessments",
                },
                "height": 8,
                "width": 6,
            },
            "accumulated_assessments": {
                "name": "Accumulated Assessments",
                "display": "scalar",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["count"]],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    }
                ],
                "visualization_settings": {},
            },
            "new_assessments_per_month": {
                "name": "New Assessments per Month",
                "display": "line",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "datetime-field",
                                [
                                    "field-id",
                                    self.tables["assessment"]["fields"]["start_date"],
                                ],
                                "month",
                            ]
                        ],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Start Date",
                        "name": "start_date",
                        "special_type": "type/CreationTimestamp",
                        "unit": "month",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.show_trendline": True,
                    "graph.y_axis.title_text": "Number of started assessments",
                    "graph.show_values": False,
                    "graph.x_axis.title_text": "Date",
                    "graph.label_value_frequency": "fit",
                    "graph.metrics": ["count"],
                    "series_settings": {"count": {"display": "bar"}},
                    "graph.dimensions": ["start_date"],
                    "stackable.stack_type": None,
                },
            },
            "completion_of_assessments": {
                "name": "Completion of Assessments",
                "display": "bar",
                "query_type": "native",
                "dataset_query": {
                    "database": self.database_id,
                    "native": {
                        "query": "select (\n"
                        "    case when completion_percentage > 70 then "
                        "'top (more than 70% of risks answered)'\n"
                        "         when completion_percentage >= 10 and completion_percentage <= 70 then 'average (more than 10% of risks answered)'\n"
                        "         when completion_percentage < 10 then 'low (less than 10% of risks answered)'\n"
                        "         when completion_percentage is null then 'unknown (no data)'\n"
                        "         else 'unknown (unusable data)'\n"
                        "end) as completion,\n"
                        "count(*) from assessment\n"
                        "where completion_percentage >= 0 "
                        "  {extra_filter} \n"
                        "group by completion "
                        "order by min(completion_percentage) desc;".format(
                            extra_filter=(
                                self.extra_filter["native"] if self.extra_filter else ""
                            )
                        )
                    },
                    "type": "native",
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "completion",
                        "name": "completion",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.y_axis.title_text": "Number of Assessments",
                    "graph.show_values": True,
                    "table.cell_column": "count",
                    "stackable.stack_display": "bar",
                    "graph.x_axis.title_text": "Completion Percentage",
                    "graph.y_axis.scale": "pow",
                    "graph.metrics": ["count"],
                    "graph.label_value_formatting": "auto",
                    "table.pivot_column": "completion",
                    "series_settings": {
                        "2": {"color": "#88BF4D"},
                        "22": {"color": "#F9D45C"},
                        "86": {"color": "#EF8C8C"},
                        "count": {"color": "#98D9D9", "display": "bar"},
                    },
                    "graph.dimensions": ["completion", "count"],
                    "stackable.stack_type": None,
                },
            },
            "accumulated_assessments_over_time": {
                "name": "Accumulated Assessments Over Time",
                "display": "line",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["cum-count"]],
                        "breakout": [
                            [
                                "datetime-field",
                                [
                                    "field-id",
                                    self.tables["assessment"]["fields"]["start_date"],
                                ],
                                "month",
                            ]
                        ],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/DateTime",
                        "display_name": "Start Date",
                        "name": "start_date",
                        "special_type": "type/CreationTimestamp",
                        "unit": "month",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.show_trendline": True,
                    "graph.y_axis.title_text": "Number of Accumulated Assessments",
                    "graph.show_values": False,
                    "graph.x_axis.title_text": "Date",
                    "graph.label_value_frequency": "fit",
                    "graph.metrics": ["count"],
                    "series_settings": {
                        "count": {"display": "bar", "title": "Number of Assessments"}
                    },
                    "graph.dimensions": ["start_date"],
                    "stackable.stack_type": None,
                },
            },
            "tools_by_accumulated_assessments": {
                "name": "Tools by Accumulated Assessments",
                "display": "row",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["assessment"]["fields"]["tool_path"],
                            ]
                        ],
                        "order-by": [["desc", ["aggregation", 0]]],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Tool Path",
                        "name": "tool_path",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.dimensions": ["tool_path"],
                    "graph.metrics": ["count"],
                    "series_settings": {"count": {"title": "Number of Assessments"}},
                },
            },
            "tools_by_assessment_completion": {
                "name": "Tools by Assessment Completion",
                "display": "bar",
                "query_type": "native",
                "dataset_query": {
                    "type": "native",
                    "native": {
                        "query": (
                            "select tool_path,\n"
                            "    count(case when completion_percentage > 70 then 'top' end) as top_assessments,\n"
                            "    count(case when completion_percentage >= 10 and completion_percentage <= 70 then 'avg' end) as avg_assessments,\n"
                            "    count(case when completion_percentage < 10 then 'low' end) as low_assessments\n"
                            "from assessment\n"
                            "where tool_path not like '%/preview'\n"
                            "group by tool_path\n"
                            "order by top_assessments desc, avg_assessments desc, low_assessments desc;"
                        ),
                        "template-tags": {},
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Tool Path",
                        "name": "tool_path",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "top_assessments",
                        "name": "top_assessments",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "avg_assessments",
                        "name": "avg_assessments",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "low_assessments",
                        "name": "low_assessments",
                        "special_type": None,
                    },
                ],
                "visualization_settings": {
                    "series_settings": {
                        "top_assessments": {
                            "color": "#88BF4D",
                            "title": "Top Assessments",
                        },
                        "low_assessments": {
                            "color": "#EF8C8C",
                            "title": "Low Assessments",
                        },
                        "avg_assessments": {"title": "Average Assessments"},
                    },
                    "stackable.stack_type": None,
                    "graph.dimensions": ["tool"],
                    "graph.metrics": [
                        "top_assessments",
                        "avg_assessments",
                        "low_assessments",
                    ],
                    "graph.show_values": False,
                    "graph.x_axis.axis_enabled": False,
                    "graph.y_axis.auto_split": False,
                },
            },
            "accumulated_assessments_per_country": {
                "name": "Accumulated Assessments per Country",
                "display": "bar",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["assessment"]["fields"].get("country", ""),
                            ]
                        ],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Country",
                        "name": "country",
                        "special_type": "type/Country",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.dimensions": ["country"],
                    "graph.metrics": ["count"],
                },
            },
            "registered_users_per_country": {
                "name": "Registered Users per Country",
                "display": "bar",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [
                            [
                                "distinct",
                                [
                                    "field-id",
                                    self.tables["assessment"]["fields"].get(
                                        "account_id", ""
                                    ),
                                ],
                            ]
                        ],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["assessment"]["fields"].get("country", ""),
                            ]
                        ],
                        "filter": [
                            "=",
                            [
                                "field-id",
                                self.tables["assessment"]["fields"].get(
                                    "account_type", ""
                                ),
                            ],
                            "converted",
                            "full",
                        ],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "description": None,
                        "semantic_type": "type/Country",
                        "coercion_strategy": None,
                        "name": "country",
                        "settings": None,
                        "field_ref": ["field", 41, None],
                        "effective_type": "type/Text",
                        "id": 41,
                        "display_name": "Country",
                        "base_type": "type/Text",
                    },
                    {
                        "display_name": "Distinct values of Account ID",
                        "semantic_type": "type/Quantity",
                        "settings": None,
                        "field_ref": ["aggregation", 0],
                        "name": "count",
                        "base_type": "type/BigInteger",
                        "effective_type": "type/BigInteger",
                    },
                ],
                "visualization_settings": {
                    "graph.y_axis.title_text": "Registered Users",
                    "graph.dimensions": ["country"],
                    "series_settings": {
                        "count": {"title": "Number of Registered Users"}
                    },
                    "graph.metrics": ["count"],
                },
            },
            "top_tools_by_number_of_assessments": {
                "name": "Top Tools by Number of Assessments",
                "display": "row",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["assessment"]["fields"]["tool_path"],
                            ]
                        ],
                        "order-by": [["desc", ["aggregation", 0]]],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Tool Path",
                        "name": "tool_path",
                        "special_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.show_trendline": True,
                    "graph.y_axis.title_text": "Number of started assessments",
                    "graph.show_values": False,
                    "graph.x_axis.title_text": "Date",
                    "graph.label_value_frequency": "fit",
                    "graph.metrics": ["count"],
                    "series_settings": {"count": {"display": "bar"}},
                    "graph.dimensions": ["tool_path"],
                    "stackable.stack_type": None,
                },
            },
            "top_assessments_by_country": {
                "name": "Top Assessments By Country",
                "display": "bar",
                "query_type": "query",
                "dataset_query": {
                    "query": {
                        "source-table": self.tables["assessment"]["id"],
                        "aggregation": [
                            [
                                "aggregation-options",
                                [
                                    "/",
                                    [
                                        "count-where",
                                        [
                                            ">",
                                            [
                                                "field-id",
                                                self.tables["assessment"]["fields"][
                                                    "completion_percentage"
                                                ],
                                            ],
                                            70,
                                        ],
                                    ],
                                    ["count"],
                                ],
                                {"display-name": "Top Assessments"},
                            ]
                        ],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["assessment"]["fields"].get("country", ""),
                            ]
                        ],
                    },
                    "type": "query",
                    "database": self.database_id,
                },
                "visualization_settings": {
                    "table.pivot": False,
                    "graph.dimensions": ["country"],
                    "graph.metrics": ["expression"],
                    "column_settings": {
                        '["name","expression"]': {"number_style": "percent"}
                    },
                },
            },
            "accumulated_number_of_users": {
                "name": "Accumulated Users Over Time",
                "display": "line",
                "query_type": "query",
                "dataset_query": {
                    "database": self.database_id,
                    "query": {
                        "source-query": {
                            "source-table": self.tables["assessment"]["id"],
                            "aggregation": [
                                [
                                    "distinct",
                                    [
                                        "field-id",
                                        self.tables["assessment"]["fields"][
                                            "account_id"
                                        ],
                                    ],
                                ]
                            ],
                            "breakout": [
                                [
                                    "datetime-field",
                                    [
                                        "field-id",
                                        self.tables["assessment"]["fields"][
                                            "start_date"
                                        ],
                                    ],
                                    "month",
                                ]
                            ],
                        },
                        "aggregation": [
                            [
                                "cum-sum",
                                ["field", "count", {"base-type": "type/Integer"}],
                            ]
                        ],
                        "breakout": [["field-literal", "start_date", "type/DateTime"]],
                    },
                    "type": "query",
                },
                "result_metadata": [
                    {
                        "base_type": "type/Date",
                        "display_name": "Start Date",
                        "name": "start_date",
                        "special_type": "type/CreationTimestamp",
                        "unit": "day",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "graph.dimensions": ["start_date"],
                    "graph.metrics": ["count"],
                    "series_settings": {
                        "sum": {"title": "Accumulated Number of Users"}
                    },
                },
            },
            "number_of_survey_responses": {
                "name": "Number of Survey Responses",
                "display": "scalar",
                "query_type": "native",
                "dataset_query": {
                    "type": "native",
                    "native": {
                        "query": 'SELECT count(*) AS "count"\nFROM "public"."company"\nWHERE needs_met is not NULL or workers_participated is not NULL or referer is not NULL or employees is not NULL or conductor is not NULL or recommend_tool is not NULL',
                        "template-tags": {},
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    }
                ],
                "visualization_settings": {"table.cell_column": "count"},
            },
            "employees": {
                "name": "Number of Employees",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"]["employees"],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Employees",
                        "name": "employees",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.colors": {
                        "1-9": "#98D9D9",
                        "10-49": "#509EE3",
                        "250+": "#7172AD",
                        "null": "#74838f",
                        "50-249": "#A989C5",
                    },
                    "pie.slice_threshold": 0.1,
                    "column_settings": {
                        '["name","count"]': {"number_style": "decimal"}
                    },
                    "pie.show_legend": True,
                    "pie.show_legend_perecent": True,
                },
            },
            "conductor": {
                "name": "Assessment conducted by",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"]["conductor"],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Conductor",
                        "name": "conductor",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.colors": {
                        "both": "#EF8C8C",
                        "null": "#74838f",
                        "staff": "#509EE3",
                        "third-party": "#7172AD",
                    },
                    "pie.slice_threshold": 0,
                    "pie.show_legend": True,
                },
            },
            "referer": {
                "name": "Learned about OiRA",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"]["referer"],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Referer",
                        "name": "referer",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.slice_threshold": 0,
                    "pie.colors": {
                        "employers-organisation": "#88BF4D",
                        "eu-institution": "#F2A86F",
                        "null": "#74838f",
                        "other": "#509EE3",
                        "health-safety-experts": "#A989C5",
                        "national-public-institution": "#EF8C8C",
                    },
                    "pie.show_legend": True,
                },
            },
            "referer_france": {
                "name": "Learned about OiRA",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "expressions": {
                            "France referer": [
                                "replace",
                                [
                                    "field",
                                    self.tables["company"]["fields"]["referer"],
                                    None,
                                ],
                                "trade-union",
                                "social-security",
                            ]
                        },
                        "breakout": [["expression", "France referer", None]],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Text",
                        "display_name": "Refer Er",
                        "name": "referer",
                        "special_type": "type/Category",
                    },
                    {
                        "display_name": "Referer",
                        "field_ref": ["expression", "France referer"],
                        "name": "France referer",
                        "base_type": "type/Text",
                        "effective_type": "type/Text",
                        "semantic_type": None,
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.slice_threshold": 0,
                    "pie.colors": {
                        "employers-organisation": "#88BF4D",
                        "eu-institution": "#F2A86F",
                        "null": "#74838f",
                        "other": "#509EE3",
                        "health-safety-experts": "#A989C5",
                        "national-public-institution": "#EF8C8C",
                    },
                    "pie.show_legend": True,
                },
            },
            "workers_participated": {
                "name": "Workers were invited",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"][
                                    "workers_participated"
                                ],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Boolean",
                        "display_name": "Workers Participated",
                        "name": "workers_participated",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.show_legend": True,
                    "pie.slice_threshold": 0,
                    "pie.colors": {
                        "null": "#74838f",
                        "true": "#88BF4D",
                        "false": "#F2A86F",
                    },
                },
            },
            "needs_met": {
                "name": "Needs were met",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"]["needs_met"],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Boolean",
                        "display_name": "Needs Met",
                        "name": "needs_met",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.slice_threshold": 0,
                    "pie.colors": {
                        "null": "#74838f",
                        "true": "#88BF4D",
                        "false": "#F2A86F",
                    },
                    "pie.show_legend": True,
                },
            },
            "recommend_tool": {
                "name": "Would recommend tool",
                "display": "pie",
                "query_type": "query",
                "dataset_query": {
                    "type": "query",
                    "query": {
                        "source-table": self.tables["company"]["id"],
                        "aggregation": [["count"]],
                        "breakout": [
                            [
                                "field-id",
                                self.tables["company"]["fields"]["recommend_tool"],
                            ]
                        ],
                    },
                    "database": self.database_id,
                },
                "result_metadata": [
                    {
                        "base_type": "type/Boolean",
                        "display_name": "Recommend Tool",
                        "name": "recommend_tool",
                        "special_type": "type/Category",
                    },
                    {
                        "base_type": "type/BigInteger",
                        "display_name": "Count",
                        "name": "count",
                        "special_type": "type/Quantity",
                    },
                ],
                "visualization_settings": {
                    "pie.show_legend": True,
                    "pie.slice_threshold": 0,
                    "pie.colors": {
                        "null": "#74838f",
                        "true": "#88BF4D",
                        "false": "#F2A86F",
                    },
                },
            },
            "oiras_poissonerie_sessions_cumulees": self._get_inrs_card("poissonerie"),
            "oiras_boulangerie_sessions_cumulees": self._get_inrs_card("boulangerie"),
            "oiras_boucherie_charcuterie_sessions_cumulees": self._get_inrs_card(
                "boucherie-charcuterie"
            ),
            "oiras_commerce_alimentaire_de_proximite_sessions_cumulees": self._get_inrs_card(
                "commerce-alimentaire-de-proximite"
            ),
        }

    def _get_inrs_card(self, tool):
        return {
            "name": "OiRA {} - Sessions cumulées".format(tool),
            "display": "line",
            "query_type": "query",
            "dataset_query": {
                "type": "query",
                "query": {
                    "source-table": self.tables["assessment"]["id"],
                    "filter": [
                        "ends-with",
                        [
                            "field",
                            self.tables["assessment"]["fields"]["tool_path"],
                            None,
                        ],
                        tool,
                        {"case-sensitive": False},
                    ],
                    "aggregation": [["cum-count"]],
                    "breakout": [
                        [
                            "field",
                            self.tables["assessment"]["fields"]["start_date"],
                            {"temporal-unit": "month"},
                        ],
                    ],
                },
                "database": self.database_id,
            },
            "result_metadata": [
                {
                    "semantic_type": "type/CreationTimestamp",
                    "coercion_strategy": None,
                    "unit": "month",
                    "name": "start_date",
                    "display_name": "Start Date",
                    "base_type": "type/DateTime",
                },
                {
                    "name": "count",
                    "display_name": "Count",
                    "base_type": "type/BigInteger",
                    "semantic_type": "type/Quantity",
                },
            ],
            "visualization_settings": {
                "graph.dimensions": ["start_date"],
                "graph.metrics": ["count"],
            },
        }


class SectorCardFactory(CardFactory):
    def __init__(self, sector_name, *args):
        self.sector_name = sector_name
        super(SectorCardFactory, self).__init__(*args)

    @property
    def extra_filter(self):
        return {
            "query": [
                "=",
                [
                    "field-id",
                    self.tables["assessment"]["fields"]["tool_path"],
                ],
            ]
            + config.sectors[self.sector_name],
            "native": " AND ({}) ".format(
                " OR ".join(
                    (
                        "tool_path = '{}'".format(path)
                        for path in config.sectors[self.sector_name]
                    )
                )
            ),
        }

    def transform_name(self, name):
        return "{} ({})".format(name, self.sector_name)



from metabase_api import Metabase_API

import json
import logging
import requests


log = logging.getLogger(__name__)


class OiraMetabase_API(Metabase_API):
    def authenticate(self):
        """Get a Session ID"""
        conn_header = {"username": self.email, "password": self.password}

        try:
            res = requests.post(
                self.domain + "/api/session", json=conn_header, timeout=15
            )
        except requests.exceptions.Timeout:
            log.warn("Authentication timed out, retrying")
            res = requests.post(
                self.domain + "/api/session", json=conn_header, timeout=30
            )
        if not res.ok:
            raise Exception(res)

        self.session_id = res.json()["id"]
        self.header = {"X-Metabase-Session": self.session_id}

    def get(self, endpoint, **kwargs):
        self.validate_session()
        result = requests.get(self.domain + endpoint, headers=self.header, **kwargs)
        self.check_error(result)
        return result

    def post(self, endpoint, **kwargs):
        self.validate_session()
        result = requests.post(self.domain + endpoint, headers=self.header, **kwargs)
        self.check_error(result)
        return result

    def put(self, endpoint, **kwargs):
        self.validate_session()
        result = requests.put(self.domain + endpoint, headers=self.header, **kwargs)
        self.check_error(result)
        return result

    def delete(self, endpoint, **kwargs):
        self.validate_session()
        result = requests.delete(self.domain + endpoint, headers=self.header, **kwargs)
        self.check_error(result)
        return result

    def check_error(self, result):
        if not result.ok:
            if result.status_code not in [404]:
                try:
                    errors = result.json().get("errors") or result.json().get("message")
                except json.decoder.JSONDecodeError:
                    errors = result.text
            else:
                errors = result.reason
            log.error(
                "Error {status_code} during {method} request to {url}! ({errors})"
                "".format(
                    method=result.request.method,
                    url=result.url,
                    status_code=result.status_code,
                    errors=errors,
                )
            )



from alembic import op
from logging import getLogger
from sqlalchemy import inspect


logger = getLogger(__name__)


def has_table(name):
    """Utility function that can be used in alembic upgrades to check if a
    table exists."""
    return name in inspect(op.get_bind()).get_table_names()


def has_column(table_name, column_name):
    """Utility function that can be used in alembic upgrades to check if a
    column exists."""
    bind = op.get_bind()
    columns = inspect(bind).get_columns(table_name)
    return any(column["name"] == column_name for column in columns)



from sqlalchemy import schema
from sqlalchemy import types
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql import functions


Base = declarative_base()


# Enum copied from euphorie.client.model so we do not have to pull it in as a dependency
class Enum(types.TypeDecorator):
    impl = types.Unicode

    def __init__(self, values, empty_to_none=False, strict=False):
        """Emulate an Enum type.

        values:
           A list of valid values for this column
        empty_to_none:
           Optional, treat the empty string '' as None
        strict:
           Also insist that columns read from the database are in the
           list of valid values.  Note that, with strict=True, you won't
           be able to clean out bad data from the database through your
           code.
        """

        if values is None or len(values) == 0:
            raise TypeError("Enum requires a list of values")
        self.empty_to_none = empty_to_none
        self.strict = strict
        self.values = values[:]

        # The length of the string/unicode column should be the longest string
        # in values
        size = max([len(v) for v in values if v is not None])
        super().__init__(size)

    def process_bind_param(self, value, dialect):
        if self.empty_to_none and value == "":
            value = None
        if value not in self.values:
            raise ValueError('"%s" not in Enum.values' % value)
        if value is None:
            return None
        else:
            return value

    def process_result_value(self, value, dialect):
        if value is None:
            return value
        if self.strict and value not in self.values:
            raise ValueError('"%s" not in Enum.values' % value)
        return value


class AccountStatistics(Base):
    """Statistically relevant data concerning an account."""

    __tablename__ = "account"

    id = schema.Column(types.Integer(), primary_key=True, autoincrement=True)
    account_type = schema.Column(
        Enum(["guest", "converted", "full"]), default="full", nullable=True
    )
    creation_date = schema.Column(
        types.DateTime, nullable=True, default=functions.now()
    )


class SurveyStatistics(Base):
    """Statistically relevant data concerning a survey (tool)."""

    __tablename__ = "tool"

    tool_path = schema.Column(types.String(512), primary_key=True)
    published_date = schema.Column(types.DateTime, nullable=True)
    years_online = schema.Column(types.Integer(), nullable=True)
    num_users = schema.Column(types.Integer(), nullable=True)
    num_assessments = schema.Column(types.Integer(), nullable=True)


class SurveySessionStatistics(Base):
    """Statistically relevant data concerning a session."""

    __tablename__ = "assessment"

    tool_path = schema.Column(types.String(512), nullable=False)
    completion_percentage = schema.Column(types.Integer, nullable=True, default=0)
    account_type = schema.Column(
        Enum(["guest", "converted", "full"]), default="full", nullable=True
    )
    start_date = schema.Column(types.DateTime, nullable=False, default=functions.now())
    modified = schema.Column(types.DateTime, nullable=True)
    country = schema.Column(types.String(512), nullable=False)
    id = schema.Column(types.Integer(), primary_key=True, autoincrement=True)
    account_id = schema.Column(types.Integer(), nullable=True)


class NewsletterStatistics(Base):
    """Statistics on the newsletter subscriptions."""

    __tablename__ = "newsletter"
    zodb_path = schema.Column(types.String(512), primary_key=True, nullable=False)
    count = schema.Column(types.Integer(), default=0)


class CompanyStatistics(Base):
    """Statistically relevant data concerning a company."""

    __tablename__ = "company"

    id = schema.Column(types.Integer(), primary_key=True, autoincrement=True)
    country = schema.Column(types.String(3))
    employees = schema.Column(Enum(["no answer", "1-9", "10-49", "50-249", "250+"]))
    conductor = schema.Column(Enum(["no answer", "staff", "third-party", "both"]))
    referer = schema.Column(
        Enum(
            [
                "no answer",
                "employers-organisation",
                "trade-union",
                "national-public-institution",
                "eu-institution",
                "health-safety-experts",
                "other",
            ]
        )
    )
    workers_participated = schema.Column(Enum(["no answer", "yes", "no"]))
    needs_met = schema.Column(Enum(["no answer", "yes", "no"]))
    recommend_tool = schema.Column(Enum(["no answer", "yes", "no"]))
    date = schema.Column(types.DateTime(), nullable=True)
    tool_path = schema.Column(types.String(512), nullable=False)



"""Add modified and completion percentage columns to survey statistics.

Revision ID: 20220921134624
Revises:
Create Date: 2022-09-21 11:42:25.215599
"""

from alembic import op
from oira.statistics.deployment.upgrade.utils import has_column

import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "20220921134624"
down_revision = None
branch_labels = None
depends_on = None


def upgrade(engine_name):
    if not has_column("assessment", "modified"):
        op.add_column("assessment", sa.Column("modified", sa.DateTime(), nullable=True))
    if not has_column("assessment", "completion_percentage"):
        op.add_column(
            "assessment",
            sa.Column("completion_percentage", sa.Integer(), nullable=True, default=0),
        )


def downgrade(engine_name):
    if has_column("assessment", "modified"):
        op.drop_column("assessment", "modified")
    if has_column("assessment", "completion_percentage"):
        op.drop_column("assessment", "completion_percentage")



# -*- coding: utf-8 -*-
sectors = {
    "Agriculture": [
        "be/agriculture/agriculture",
        "be/landbouw/landbouw-1",
        "gr/georgia/georgia",
        "es/agricultura/agricultura-c",
        "es/agricultura/agricultura-e",
        "is/agriculture/agriculture",
        "pt/agricultura/agricultura",
        "lt/lt-test-sector/zemes-ukis",
        "lv/latvia-test/lauksaimnieciba",
        "eu/eu-agriculture/oira-agriculture",
    ],
    "COVID": [
        "si/covid-19/covid-19",
        "hr/covid-19/covid-19",
        "gr/covid-19/covid-19",
        "be/covid-19/covid-19-fr",
        "be/covid-19/covid-19-nl",
        "cz/covid-19/covid-19-cz",
        "lv/latvia-test/covid-19",
        "lt/covid-19/covid-19",
        "bg/covid-19/covid-19",
        "fr/covid/covid",
        "pt/covid-19/covid-19",
        "cy/covid-19/covid-19",
        "es/covid/covid",
        "es/covid/covid-19-es",
        "eu/covid-19/covid-19",
    ],
    "Cleaning": [
        "cz/uklidovu-pracovnik/uklidovu-pracovnik",
        "eu/eu-cleaning/eu-cleaning",
        "be/schoonmaak/schoonmaak",
        "be/schoonmaak/nettoyage-classique",
        "lt/lt-test-sector/misko-ruosa",
        "si/ciscenje-poslovnih-prostorov/ciscenje-poslovnih-prostorov-1",
        "lv/latvia-test/uzkopsanas-darbi",
    ],
    "Education": [
        "es/educacion/secundaria",
        "es/educacio/secundaria",
        "es/educacio/preescolar",
        "es/educacion/preescolar",
        "hr/njegovatelji-djece/njegovatelji-djece-m1",
        "si/ucitelj/ucitelj",
        "eu/eu-education/oira-early-childhood-education-and-care",
        "eu/eu-education/oira-secondary-education",
        "gr/ekpaideutikes-monades/educational-premises-gr",
        "lt/lt-test-sector/svietimo-istaigos",
        "es/autoescuelas/autoescuelas",
        "lv/latvia-test/izglitibas-iestades-1",
        "cy/schools2015/scholikes-monades",
        "bg/obrazovanie-1/obrazovanie",
    ],
    "Hairdressers": [
        "fr/soins-esthetiques/soins-esthetiques",
        "fr/coiffure/coiffure",
        "lt/lt-test-sector/kirpyklos-ir-grozio-salonai",
        "cy/hairdressers/koureia-kommoteria-cy-hairdressing_v2",
        "hr/frizerski-salon/frizerski-salon-1",
        "eu/eu-hairdressing/eu-hairdressing",
        "is/icelandic-sector-leifur/harsnyrtistofur-1-desember-2016",
        "es/perruqueries/peluquerias",
        "gr/hairdressers/hairdressing-salons",
        "cz/kadernicke-sluzby/kadernicke-sluzby",
        "pt/cabeleireiro/cabeleireirofinal",
        "be/coiffure/oira-coiffure",
        "be/coiffure/oira-kappers",
        "si/frizer/delo-v-frizerski-dejavnosti",
    ],
    "HORECA": [
        "be/oira-horeca/oira-horeca",
        "be/oira-horeca/oira-horeca-test-nl",
        "bg/khotieli-i-riestoranti/khotieli-i-riestoranti",
        "cy/etaireies-trophodosias-catering/klados-etaireion-trophodosias-catering",
        "cy/kaphestiatoria-1/kaphestiatoria",
        "cy/mikra-touristika-katalumata/mikra-touristika-katalumata",
        "es/horeca/horeca",
        "es/horeca/horeca-1",
        "eu/eu-horeca/oira-horeca",
        "fr/hotellerie-restauration/hcr",
        "fr/restauration-rapide/restauration-rapide/",
        "fr/traiteurs-et-organisateurs-de-reception/traiteurs-et-organisateurs-de-reception/",
        "gr/catering/ergaleio-oira-gia-tin-ektimisi-kindynoy-ston-klado-ton-etaireion-trofodosias-catering",
        "gr/kapheteries-kaphe-estiatoria/kapheteries-kaphe-estiatoria-ekd-3e-iounios-2022",
        "hr/konobar/konobar",
        "hr/kuhar/kuharica",
        "hr/radnik-u-cateringu/radnik-u-cateringu",
        "is/catering/veitingar-sept-2015",
        "lt/lt-test-sector/viesbuciai-ir-restoranai",
        "lv/latvia-test/sabiedriska-edinasana",
        "lv/latvia-test/viesnicas-un-viesu-majas",
        "mt/catering/catering-mt",
        "pt/restauracao/restauracao",
        "si/kuhar/kuhar",
        "si/natakar/natakar",
    ],
    "Leather and tanning": [
        "lv/eu/leather-tanning/leather-and-tanning-version-2",
        "pt/curtumes/curtumes",
        "es/adoberies/curtiduria",
        "es/adoberies/adoberies",
        "bg/obrabotka-na-kozhi-proizvodstvo-na-obuvki/obrabotka-na-kozhi-proizvodstvo-na-obuvki",
        "eu/leather-tanning/leather-tanning-draft-risk-assessment-tool",
    ],
    "Live performance": [
        "fi/esittava-taide/esittava-taide",
        "gr/epikheireseis-khoron-parousiases-zontanon-ekdeloseon/epikheireseis-khoron-parousiases-zontanon-ekdeloseon",
        "gr/epikheireseis-paragoges-zontanon-ekdeloseon-1/epikheireseis-khoron-parousiases-zontanon-ekdeloseon",
        "be/podiumkunsten-locaties-nl-preview/podiumkunsten-locaties-nl-preview",
        "be/podia/podiumkunsten-fr-preview-08012018",
        "be/podiumkunsten/oira-podiumkunsten",
        "be/podiumkunsten-locaties-fr-preview/podiumkunsten-locaties-fr-preview",
        "eu/live-performance/eu-representaciones-en-vivo-recintos-y-salas",
        "eu/live-performance/ue-representaciones-en-vivo-producciones",
        "eu/live-performance/live-performance-oira-tool_venues",
        "eu/live-performance/live-performance-oira-tool_productions",
        "is/live-performance/lifandi-flutningur-svidslistir",
    ],
    "Private security": [
        "es/seguretatprivada/seguridad-privada",
        "gr/private-security-sector/etaireies-parokhes-uperesion-asphaleias",
        "es/seguretatprivada/seguretat-privada",
        "eu/eu-private-security/private-security-eu",
    ],
    "Sports": [
        "lv/latvia-test/sporta-centrs",
        "eu/eu-sport-sector/eu-sport-sector-active-leisure",
        "eu/eu-sport-sector/eu-sport-sector-not-for-profit",
        "eu/eu-sport-sector/eu-sport-sector-professional-sports",
    ],
}



from setuptools import find_packages
from setuptools import setup


version = "1.1.0"

setup(
    name="oira.statistics.deployment",
    version=version,
    description="Deployment helpers for the statistics of the OSHA-OiRA site.",
    long_description=open("README.md").read() + "\n" + open("CHANGES.txt").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="euphorie osha oira",
    author="syslab.com",
    author_email="info@syslab.com",
    url="http://www.oiraproject.eu/",
    license="GPL",
    packages=find_packages("src"),
    package_dir={"": "src"},
    namespace_packages=["oira", "oira.statistics"],
    include_package_data=True,
    python_requires="~= 3.8",
    install_requires=[
        "alembic",
        "metabase-api",
        "setuptools",
        "SQLAlchemy[postgresql] >=1.2.999999",
    ],
    entry_points="""
    [console_scripts]
    init-metabase-instance = oira.statistics.deployment.scripts:init_metabase_instance
    """,
)



"""Add newsletter statistics table

Revision ID: 20230228134447
Revises: 20220921134624
Create Date: 2023-02-28 13:48:49.647245

"""

from alembic import op
from oira.statistics.deployment.upgrade.utils import has_table

import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "20230228134447"
down_revision = "20220921134624"
branch_labels = None
depends_on = None


def upgrade(engine_name):
    if not has_table("newsletter"):
        op.create_table(
            "newsletter",
            sa.Column("zodb_path", sa.String(length=512), nullable=False),
            sa.Column("count", sa.Integer(), nullable=True),
            sa.PrimaryKeyConstraint("zodb_path"),
        )


def downgrade(engine_name):
    if has_table("newsletter"):
        op.drop_table("newsletter")
